package cn.com.qmth.examcloud.core.examwork.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.persistence.criteria.Predicate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.examwork.dao.ExamSpecialSettingsRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamSpecialSettingsEntity;
import cn.com.qmth.examcloud.core.examwork.service.OnGoingExamService;

@Service
public class OnGoingExamServiceImpl implements OnGoingExamService {

	@Autowired
	ExamSpecialSettingsRepo examSpecialSettingsRepo;

	@Override
	public List<ExamSpecialSettingsEntity> getOngoingExamList(Long rootOrgId, String examType,
			Long orgId, Long studentId) {

		ExamType et = null;
		if (StringUtils.isNotBlank(examType)) {
			try {
				et = ExamType.valueOf(examType);
			} catch (Exception e) {
				throw new StatusException("002005", "examType is wrong");
			}
		}

		final ExamType type = et;

		Specification<ExamSpecialSettingsEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));
			predicates.add(cb.greaterThan(root.get("endTime"), new Date()));
			if (null != type) {
				predicates.add(cb.equal(root.get("examType"), type));
			}

			predicates.add(cb.equal(root.get("examEnable"), true));
			predicates.add(cb.isNull(root.get("courseId")));

			if (null != orgId) {
				Predicate pr1 = cb.isNull(root.get("orgId"));
				Predicate pr2 = cb.equal(root.get("orgId"), orgId);
				predicates.add(cb.or(pr1, pr2));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = PageRequest.of(0, 10000, new Sort(Direction.ASC, "beginTime"));

		List<ExamSpecialSettingsEntity> entityList = examSpecialSettingsRepo
				.findAll(specification, pageRequest).getContent();

		List<ExamSpecialSettingsEntity> resultList = Lists.newArrayList();
		Map<String, ExamSpecialSettingsEntity> examMap = Maps.newHashMap();
		for (ExamSpecialSettingsEntity cur : entityList) {
			if (null == cur.getOrgId() && null == cur.getCourseId()) {
				examMap.put(String.valueOf(cur.getExamId()), cur);
			}
			if (null != cur.getBeginTime() && null != cur.getEndTime()) {
				resultList.add(cur);
			}
		}
		for (ExamSpecialSettingsEntity cur : entityList) {
			if ((null != cur.getOrgId() || null != cur.getCourseId()) && null != cur.getEndTime()) {
				ExamSpecialSettingsEntity tmp = examMap.get(String.valueOf(cur.getExamId()));
				if (null != tmp) {
					resultList.remove(tmp);
				}
			}
		}

		return resultList;
	}

}
